package com.zsc.edu.gateway.modules.system.utils;


import com.zsc.edu.gateway.exception.NotExistException;
import com.zsc.edu.gateway.modules.system.entity.Dept;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Yao
 */
public class DeptTreeUtil {

    public static Dept initTree(HashSet<Dept> list){
        Dept rootDept = list.stream()
                //                .filter(dept -> dept.id.equals(deptId))
                .filter(dept -> dept.getPid() == null)
                .findFirst()
                .orElseThrow(NotExistException::new);
        rootDept.setChildren(DeptTreeUtil.getTree(list, rootDept.getId()));
        return rootDept;
    }

    public static Set<Dept> initTrees(List<Dept> list){
        Map<Long, Dept> deptMap = list.stream().collect(Collectors.toMap(Dept::getId, dept -> dept));
        HashSet<Dept> result = new HashSet<>();
        list.forEach(dept -> {
            Dept parent = deptMap.get(dept.getPid());
            if (Objects.nonNull(parent)) {
                parent.getChildren().add(dept);
            } else {
                result.add(dept);
            }
        });
        return result;
    }

    public static HashSet<Dept> getTree(HashSet<Dept> list, long parentId){
        //获取所有子节点
        HashSet<Dept> childTreeList = getChildTree(list, parentId);
        for (Dept dept:childTreeList) {
            dept.setChildren(getTree(list, dept.getId()));
        }
        return childTreeList;
    }

    public static HashSet<Dept> getChildTree(HashSet<Dept> list, long id){
        HashSet<Dept> childTree = new HashSet<>();
        for (Dept dept: list) {
            if(dept.getPid() != null && dept.getPid() == id){
                childTree.add(dept);
            }
        }
        return childTree;
    }

    /**
     * 从部门树中获取子树
     * @param list
     * @return
     */
    public static Dept getChildNode(Set<Dept> list, long id) {
        for (Dept child: list) {
            if (child.getId() == id) {
                return child;
            } else if (child.getChildren() != null && !child.getChildren().isEmpty()) {
                return getChildNode(child.getChildren(), id);
            }
        }
        return null;
    }

    /**
     * 从部门获取本身及子部门的所有id
     * @param dept
     * @return
     */
    public static HashSet<Long> getDeptPath(Dept dept) {
        HashSet<Long> result = new HashSet<>();
        result.add(dept.getId());
        Stack<Dept> deptStack = new Stack<>();
        Set<Dept> children = dept.getChildren();
        if (!children.isEmpty()) {
            deptStack.addAll(children);
            while (!deptStack.isEmpty()) {
                Dept child = deptStack.pop();
                result.add(child.getId());
                if (!child.getChildren().isEmpty()) {
                    deptStack.addAll(child.getChildren());
                }
            }
        }
        return result;
    }
}
